# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

"""
.. module:: sunlight.errors
    :synopsis: Exceptions and Errors
"""

class SunlightException(Exception):
    """
    :class:`sunlight.errors.SunlightException` is the base exception,
    all other sunlight exceptions (such as
    :class:`sunlight.errors.NoAPIKeyException`) all inherit from this. This
    makes it easy to catch all sunlight errors if you really really need to
    (not that you should)
    """
    def __init__(self, value):
        """
        This is just your basic __init__ method, nothing special here.

        Args:
            value (str): Message to report with the Exception
        """
        self.value = value

    def __str__(self):
        """
        Return the string-ular representation of the exception - this makes it
        super easy to just run something like ``print e`` (given ``e`` is a
        SunlightException instance)
        """
        return repr(self.value)

class BadRequestException(SunlightException):
    """
    This gets thrown when the underlying url request has recieved an abnormal
    response code, or the program has issued a request that can not be filled.
    """
    pass


class InvalidRequestException(BadRequestException):
    """
    This gets thrown when the API gets a valid response, but has an error that
    should be passed back to the program.
    """
    pass

class NoAPIKeyException(SunlightException):
    """
    This gets thrown if the bindings are asked to issue a request, but the
    ``sunlight.config.API_KEY`` variable is ``None``.
    """
    pass



# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

"""
.. module:: sunlight
    :synopsis: Sunlight API Entry Point

Sunlight API __init__ entry. This package includes and defines
a few aliases, so that folks can just use the facade here, rather then
fight with finding the right module to use.
"""

__appname__ = "sunlight"
__version__ = "1.1.5"

import sunlight.services.openstates
import sunlight.services.capitolwords
import sunlight.services.congress
import sunlight.services.influenceexplorer

openstates = sunlight.services.openstates.openstates()
capitolwords = sunlight.services.capitolwords.capitolwords()
congress = sunlight.services.congress.congress()
influenceexplorer = sunlight.services.influenceexplorer.InfluenceExplorer()

import os.path
import warnings
import sunlight.config
import sunlight.service

def _attempt_to_load_apikey():
    """
    This function (which will be auto-called on import of :mod:`sunlight`),
    will attempt to pull the Sunlight API Key from a few places (to offload
    complexity of apps using these bindings) before getting too far.

    .. note::
        This function is implemented to let the enviroment variable override
        the file read key. Keep this in mind when debugging silly issues.
    """

    try:
        fp = os.path.expanduser(sunlight.config.KEY_LOCATION)
        fd = open(fp, 'r')
        sunlight.config.API_KEY = fd.read().strip()
    except IOError as e:
        if e.errno != 2:
            warnings.warn('key file %s exists but could not be opened: %s' % (
                sunlight.config.KEY_LOCATION, str(e)))
    try:
        sunlight.config.API_KEY = \
                os.environ[sunlight.config.KEY_ENVVAR].strip()
    except KeyError as e:
        pass

_attempt_to_load_apikey()



# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

"""
.. module:: sunlight.services.influenceexplorer
    :synopsis: InfluenceExplorer API Implementation

InfluenceExplorer API Implementation inside ``python-sunlight``.
"""

from sunlight.errors import InvalidRequestException

import sunlight.service
import json

service_url = "http://transparencydata.com/api/1.0"

class InfluenceExplorer(sunlight.service.Service):
    """
    Bindings into the `InfluenceExplorer <http://influenceexplorer.com/>`_
    project. Be careful to note that this is all a very thing layer over the
    actual API it's self, so be sure to consult the
    `docs <http://data.influenceexplorer.com/api/>`_ when in doubt over a
    method's usage.
    """

    def contributions(self, **kwargs):
        """
        Query the InfluenceExplorer server for information regarding political
        campaign contributions.

        The docs for this method can be found in the
        `docs <http://data.influenceexplorer.com/api/contributions/>`_.
        """
        return self.get("contributions", **kwargs)

    def lobbying(self, **kwargs):
        """
        Query the InfluenceExplorer server for information regarding federal
        lobbying.

        The docs for this method can be found in the
        `docs <http://data.influenceexplorer.com/api/lobbying/>`_
        """
        return self.get("lobbying", **kwargs)

    def grants(self, **kwargs):
        """
        Query the InfluenceExplorer server for information regarding federal
        grants.

        The docs for this method can be found in the
        `docs <http://data.influenceexplorer.com/api/grants/>`_
        """
        return self.get("grants", **kwargs)

    def contracts(self, **kwargs):
        """
        Query the InfluenceExplorer server for information regarding federal
        contracts.

        The docs for this method can be found in the
        `docs <http://data.influenceexplorer.com/api/contracts/>`_
        """
        return self.get("contracts", **kwargs)

    def entities(self, **kwargs):
        """
        Query the InfluenceExplorer server for information regarding aggregate
        contributions by a given entity.

        It sounds much more confusing then it really is.

        The docs for this method can be found in the
        `docs <http://data.influenceexplorer.com/api/aggregates/contributions/>`_
        """
        return self.get("entities", **kwargs)

    # API impl methods below

    def _get_url( self, obj, apikey, **kwargs ):
        return "%s/%s?apikey=%s&%s" % (
            service_url,
            obj,
            apikey,
            sunlight.service.safe_encode(kwargs)
        )

    def _decode_response( self, response ):
        ret = json.loads( response )
        if "error" in ret:
            ex = InvalidRequestException( ret['error'] )
            ex.response = ret
            raise ex
        return ret




# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.


from sunlight.errors import InvalidRequestException, BadRequestException

import sunlight.service
import json

service_url = "http://capitolwords.org/api"

class capitolwords(sunlight.service.Service):
    """
    Bindings into the `CapitolWords project <http://capitolwords.org>`_.
    Keep in mind, as you use this wrapper, that this is a very thin translation
    layer from the API into Python. There's no object magic, so the docs on the
    API it's self will be as useful as any of these docstrings.

    Feel free to read up on the CapitolWords API in the
    `docs <http://capitolwords.org/api/>`_.

    If a request goes through and gets a normal response code, but the requested
    action is for whatever reason errorful, a
    :class:`sunlight.errors.InvalidRequestException` will be thrown with the
    error string.
    """

    def dates(self, phrase, **kwargs):
        """
        Retrieve information regarding the popularity of a phrase over a
        period of time.

        For a list of optional arguments see `Capitol Words' dates.json
        endpoint <http://capitolwords.org/api/#dates.json>`_.
        """
        kwargs['phrase'] = phrase
        return self.get( "dates", **kwargs )

    def phrases(self, entity_type, entity_value, **kwargs):
        """
        Retrieve information on the top phrases for an facet.

        A facet can be any date, month, state or legislator.

        For a list of arguments see `Capitol Words' phrases.json
        endpoint <http://capitolwords.org/api/#phrases.json>`_.
        """
        kwargs['entity_type']  = entity_type
        kwargs['entity_value'] = entity_value

        return self.get( "phrases", **kwargs )

    def phrases_by_entity(self, entity_type, **kwargs):
        """
        Get listing of top facets for a given phrase.

        A facet can be any date, month, state or legislator.

        For a list of arguments see `Capitol Words' phrases/entity.json
        endpoint <http://capitolwords.org/api/#phrases/entity.json>`_.
        """
        lss = "%s/%s" % ("phrases", entity_type)
        return self.get( lss, **kwargs )

    def text( self, phrase=None, title=None, **kwargs ):
        """
        Full text-search against Capitol Words data.

        Neither ``phrase`` nor ``title`` are required, but one or the other
        must be used. If both are omitted, a
        :class:`sunlight.errors.BadRequestException` will be thrown.

        ``phrase`` will do a full-text search on that phrase, whereas
        ``title`` will doa  full-text search on the title of each CR
        (Congressional Record) document.

        For a list of arguments see `Capitol Words' text.json
        endpoint <http://capitolwords.org/api/#text.json>`_.
        """
        if not phrase and not title:
            raise BadRequestException(
                "You must provide one of phrase or title to this method.")

        if phrase:
            kwargs['phrase'] = phrase

        if title:
            kwargs['title']  = title

        return self.get( "text", **kwargs )

    # API impl methods below

    def _get_url( self, obj, apikey, **kwargs ):
        ret = "%s/%s.json?apikey=%s&%s" % (
            service_url,
            obj,
            apikey,
            sunlight.service.safe_encode(kwargs)
        )
        return ret

    def _decode_response( self, response ):
        ret = json.loads( response )
        if "error" in ret:
            ex = InvalidRequestException( ret['error'] )
            ex.response = ret
            raise ex
        if "results" in ret:
            return ret['results']
            # XXX: Verify this is actually
            #      what we want.
        return ret




# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

"""
.. module:: sunlight
    :synopsis: Constants and other Globally read things

.. warning::
    If you're using these directly in your app, you might be doing something
    wrong.
"""


API_KEY = None
"""
This might be populated from :func:`sunlight.attempt_to_load_apikey`, or
``None`` (as it is out of the box). All :class:`sunlight.service.Service`
objects will make use of this API key (once, in it's __init__, not after that)
to do their job.

.. note::
    All Sunlight services share API keys. Nice, right?
"""

API_SIGNUP_PAGE = "http://services.sunlightlabs.com/accounts/register/"
"""
This is a link to the API Key signup page - so that we can sanely direct people
to register for a key (if they don't already have one) -- after all, signing up
for a Sunlight API key is fun for the whole family!
"""

KEY_LOCATION    = "~/.sunlight.key"
"""
This is the location of the api key that's stored on the filesystem. Currently,
it uses a file directly under a tilde, so that windows users don't have to feel
as much pain when using the API. Usually this is something like
``~/.sunlight.key``
"""

KEY_ENVVAR      = "SUNLIGHT_API_KEY"
"""
This is the name of the ``os.environ`` key to look for. It's usually something
stupid simple, like ``SUNLIGHT_API_KEY``.
"""



#!/usr/bin/env python
# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

from sunlight import __appname__, __version__
from setuptools import setup

long_description = open('README.rst').read()

setup(
    name       = __appname__,
    version    = __version__,
    packages   = [ 'sunlight', 'sunlight.services' ],

    author       = "Paul Tagliamonte",
    author_email = "paultag@sunlightfoundation.com",

    long_description = long_description,
    description      = 'Unified Sunlight API bindings',
    license          = "BSD",
    url              = "https://github.com/sunlightlabs/python-sunlight",

    platforms        = ['any']
)



# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

import sunlight.service
from sunlight.errors import BadRequestException
import json

module_name = "openstates"
service_url = "http://openstates.org/api/v1"

class openstates(sunlight.service.Service):
    """
    Bindings into the `Open States API <http://openstates.org/api/>`_. Keep in
    mind this is a thin wrapper around the API so the API documentation is the
    place to look for help on field names and examples.
    """

    def all_metadata(self, **kwargs):
        """
        Get an overview of all available states including each state's name
        and abbreviation.

        For details see
        `Metadata Overview <http://openstates.org/api/metadata/#metadata-overview>`_.
        """
        return self.get(["metadata"], **kwargs)

    def state_metadata(self, state, **kwargs):
        """
        Complete metadata for a given state, containing information on
        the particulars of this state's chambers, sessions, and terms.

        For details see
        `State Metadata <http://openstates.org/api/metadata/#state-metadata>`_.
        """
        return self.get(["metadata", state], **kwargs)

    def bills(self, **kwargs):
        """
        Search the entirety of bills available via the Open States API.

        The fields and keyword arguments can be found on the
        `Bill API docs <http://openstates.org/api/bills/>`_.
        """
        return self.get(["bills"], **kwargs)

    def bill(self, bill_id, **kwargs):
        """
        Get full information on a single bill from the Open States API given
        the OpenStates Bill ID (such as TNB00004685)

        `Bill API docs <http://openstates.org/api/bills/>`_.
        """
        return self.get(["bills", bill_id], **kwargs)

    def bill_detail(self, state, session, bill_id, chamber=None):
        """
        Get full information on a single bill from the Open States API given
        a ``state``, ``session``, and ``bill_id`` (and optionally ``chamber``
        if the request would be ambiguous without one).

        The fields and keyword arguments can be found on the
        `Open States Bill API docs <http://openstates.org/api/bills/>`_.
        """
        lss = ["bills", state, session]
        if chamber:
            lss.append(chamber)
        lss.append(bill_id)
        return self.get(lss)

    def legislators(self, **kwargs):
        """
        Search the entirety of legislators available via the Open States API.

        The fields and keyword arguments can be found on the
        `Legislator API docs <http://openstates.org/api/legislators/>`_.
        """
        return self.get( [ "legislators"], **kwargs )

    def legislator_detail(self, leg_id, **kwargs):
        """
        Get detailed information on a single legislator given their Open States
        Legislator ID.

        The ``leg_id`` argument is a legislator ID code used throughout the
        Open States API, such as ``MDL000210``.

        For details on fields see the `Legislator API Fields
        <http://openstates.org/api/legislators/#legislator-fields>`_.
        """
        return self.get(["legislators", leg_id], **kwargs)

    def legislator_geo_search(self, latitude, longitude, **kwargs):
        """
        Given a latitude and longitude return all legislators that represent
        districts containing that point.

        See the Open States documentation for examples of `Legislator Geo
        Lookup <http://openstates.org/api/legislators/#geo-lookup>`_.
        """
        kwargs['lat']  = latitude
        kwargs['long'] = longitude
        return self.get(["legislators", "geo"], **kwargs)

    def committees(self, **kwargs):
        """
        Search against all committees available via the Open States API.

        Committee fields and keyword arguments can be found on the
        `Committee API docs <http://openstates.org/api/committees/>`_.
        """
        return self.get(["committees"], **kwargs)

    def committee_detail(self, committee_id, **kwargs):
        """
        Get detailed information on a single committee given its Open States
        Committee ID.

        The ``committee_id`` argument is a committee ID code used throughout
        the Open States API, such as ``MDC000065``.

        For details on fields see the `Committee API Fields
        <http://openstates.org/api/committees/#committee-fields>`_.
        """
        return self.get(["committees", committee_id], **kwargs)

    def events(self, **kwargs):
        """
        Query the Open States API for information regarding upcoming events
        taken from a state-level legislative calendar.

        See the Open States' site for details on the
        `Event API <http://openstates.org/api/events/>`_.
        """
        return self.get([ "events" ], **kwargs)

    def event_detail(self, event_id, **kwargs):
        """
        Get detailed informaton regarding a single event.

        ``event_id`` is an OpenStates event ID, such as ``TXE00000990``.

        See the Open States' site for details on the
        `Event API Fields <http://openstates.org/api/events/#event-fields>`_.
        """
        lss = [ "events", event_id ]
        return self.get( lss, **kwargs )

    def districts(self, state, chamber=None, **kwargs):
        """
        Get a listing of districts for a state (optionally narrowed by
        chamber).

        For a listing of fields see `District Fields
        <http://openstates.org/api/districts/#district-fields>`_.
        """
        pieces = ['districts', state]
        if chamber:
            pieces.append(chamber)
        return self.get(pieces, **kwargs)

    def district_boundary(self, boundary_id, **kwargs):
        """
        Get a detailed GeoJSON-style boundary for a given district given a
        boundary_id (available via the :meth:``districts``.

        ``boundary_id`` resembles ``sldl-tx-state-house-district-35``.

        For a listing of fields see `District Fields
        <http://openstates.org/api/districts/#district-fields>`_.

        For more information on this method and example output see
        `District Boundary Lookup
        <http://openstates.org/api/districts/#district-boundary-lookup>`_
        """
        return self.get(['districts', 'boundary', boundary_id], **kwargs)

    # API impl methods

    def _get_url(self, objs, apikey, **kwargs):
        # Gate for any None's in the query. This is usually a problem.
        if None in objs:
            raise BadRequestException("`None' passed to the URL encoder (%s)" %
                                      (str(objs)))

        # join pieces by slashes and add a trailing slash
        object_path = "/".join(objs)
        object_path +=  "/"

        return "%s/%s?apikey=%s&%s" % (
            service_url,
            object_path,
            apikey,
            sunlight.service.safe_encode(kwargs)
        )

    def _decode_response(self, response):
        return json.loads(response)



# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

"""
.. module:: sunlight.service
    :synopsis: Sunlight API Superclass

Base service class. All API classes (such as say -
:class:`sunlight.services.openstates.OpenStates`) inherit from this.
"""
import sys

import sunlight.config
import sunlight.errors

if sys.version_info[0] >= 3:
    from urllib.parse import urlencode
    from urllib.request import urlopen
    from urllib.error import HTTPError
    _str_type = str
else:
    from urllib import urlencode
    from urllib2 import urlopen
    from urllib2 import HTTPError
    _str_type = basestring


def safe_encode(kwargs):
    kwargs = kwargs.copy()
    for k, v in kwargs.iteritems():
        if isinstance(v, _str_type):
            kwargs[k] = v.encode('utf8')
    return urlencode(kwargs)


class Service:
    """
    Base class for all the API implementations, as well as a bunch of common
    code on how to actually fetch text over the network.
    """

    def get(self, top_level_object, **kwargs):
        """
        Get some data from the network - this is where we actually fetch
        something and make a request.

        .. warning:: Be sure that API_KEY was set before calling this method.
            This will throw a :class:`sunlight.errors.NoAPIKeyException` if
            the API_KEY is not set.

        args:
            ``top_level_object`` (str): Thing to query for (such as say,
                "bills" for OpenStates )

        kwargs:
            These arguments will be passed to the underlying API implementation
            to help create a query. Validation will happen down below, and
            on a per-API level.
        """
        if not sunlight.config.API_KEY:
            raise sunlight.errors.NoAPIKeyException(
"Warning: Missing API Key. please visit " + sunlight.config.API_SIGNUP_PAGE +
" to register for a key.")

        url = self._get_url(top_level_object, sunlight.config.API_KEY,
                            **kwargs)
        try:
            r = urlopen(url)
            return_data = r.read().decode('utf8')
            return self._decode_response(return_data)
        except HTTPError as e:
            message = e.read()
            code = e.getcode()

            ex = sunlight.errors.BadRequestException("Error (%s) -- %s" % (
                code, message
            ))

            ex.url = e.geturl()
            ex.message = message
            ex.code    = code

            raise ex



# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

"""
.. module:: sunlight.services.congress
    :synopsis: Sunlight Congress API Implementation

Sunlight Congress API Implementation inside ``python-sunlight``.
"""

import sunlight.service
import json

service_url = "http://services.sunlightlabs.com/api/"

def _unpack(resp, key):
    return [e[key] for e in resp[key+'s']]

class congress(sunlight.service.Service):
    """
    Bindings into the `Sunlight Congress API
    <http://services.sunlightlabs.com/docs/Sunlight_Congress_API/>`_, an API
    primarily for details on current federal legislators.
    """

    def legislators(self, **kwargs):
        """
        Query for all legislators matching certain criteria.

        See documentation at `legislators.get(List)
        <http://services.sunlightlabs.com/docs/congressapi/legislators.get(List)/>`_
        """
        return _unpack(self.get('legislators.getList', **kwargs),
                       'legislator')


    def legislator_search(self, name, threshold=0.9, all_legislators=False):
        """
        Fuzzy-matching name search against federal legislators.

        See documentation at `legislators.search
        <http://services.sunlightlabs.com/docs/congressapi/legislators.search/>`_
        """
        params = {'name': name, 'threshold': threshold}
        if all_legislators:
            params['all_legislators'] = 1
        return _unpack(self.get('legislators.search', **params),
                       'result')


    def legislators_for_zip(self, zipcode):
        """
        Query for all legislators representing a given ZIP code.

        This method is not recommended, prefer legislators_for_lat_lon instead.
        See the blog post `"Don't Use Zip Codes Unless You Have To"
        <http://sunlightlabs.com/blog/2012/dont-use-zipcodes/>`_.

        See documentation at `legislators.allForZip
        <http://services.sunlightlabs.com/docs/congressapi/legislators.allForZip/>`_
        """
        return _unpack(self.get('legislators.allForZip', zip=zipcode),
                       'legislator'
                      )

    def legislators_for_lat_lon(self, latitude, longitude):
        """
        Query for all legislators representing an given location.

        See documentation at `legislators.allForLatLong
        <http://services.sunlightlabs.com/docs/congressapi/legislators.allForLatLong/>`_
        """
        return _unpack(self.get('legislators.allForLatLong', latitude=latitude,
                                longitude=longitude), 'legislator')

    def districts_for_zip(self, zipcode):
        """
        Query for all congressional districts overlapping a zip code.

        See documentation at `districts.getDistrictFromLatLong
        <http://services.sunlightlabs.com/docs/congressapi/districts.getDistrictFromLatLong/>`_
        """
        return _unpack(self.get('districts.getDistrictsFromZip', zip=zipcode),
                       'district'
                      )

    def districts_for_lat_lon(self, latitude, longitude):
        """
        Query for all congressional districts containing a given location.

        See documentation at `districts.getDistrictFromLatLong
        <http://services.sunlightlabs.com/docs/congressapi/districts.getDistrictFromLatLong/>`_
        """
        return _unpack(self.get('districts.getDistrictFromLatLong',
                                latitude=latitude, longitude=longitude),
                       'district'
                      )

    def committees(self, chamber):
        """
        Query for all committees for a chamber.  (House|Senate|Joint)

        See documentation at `committees.getList
        <http://services.sunlightlabs.com/docs/congressapi/committees.getList/>`_
        """
        return _unpack(self.get('committees.getList', chamber=chamber),
                       'committee')

    def committee_detail(self, id):
        """
        Query for all details for a committee, including members.

        See documentation at `committees.get
        <http://services.sunlightlabs.com/docs/congressapi/committees.get/>`_
        """
        return self.get('committees.get', id=id)['committee']

    def committees_for_legislator(self, bioguide_id):
        """
        Query for all details for all of a legislator's committee assignments.

        See documentation at `committees.allForLegislator
        <http://services.sunlightlabs.com/docs/congressapi/committees.allForLegislator/>`_
        """
        return _unpack(self.get('committees.allForLegislator',
                                bioguide_id=bioguide_id), 'committee')

    # implementation methods
    def _get_url(self, obj, apikey, **kwargs):
        return "%s/%s?apikey=%s&%s" % (
            service_url,
            obj,
            apikey,
            sunlight.service.safe_encode(kwargs)
        )

    def _decode_response(self, response):
        return json.loads(response)['response']
